
def pullNodes(label, connection, db_name, limit=25):
    query = f"""
    MATCH (n:{label})
    RETURN n
    LIMIT {limit}
    """
    nodes = []
    
    # Execute the query
    with connection.session(database=db_name) as session:
        result = session.run(query)
        
        # Extract nodes into a list of dictionaries
        nodes = [{'id': record['n'].id, **record['n']._properties} for record in result]
        
    return nodes

    
# Fetch all nodes of a label
def get_all_nodes(label, session, limit=250):
    query = f"MATCH (result:{label}) RETURN result limit {limit}"
    result = session.run(query)
    nodes = []
    for record in result:
        node = record["result"]
        # Directly access the frozenset of labels
        node_labels = node.labels
        node_label = next(iter(node_labels), None)  # Get the first label if available
        nodes.append({"id": node.id, "properties": dict(node.items()), "label": node_label})
    return nodes


def get_related_nodes(current, nodeId, relationship, direction, db_name, driver):
    with driver.session(database=db_name) as session:
        # Adjust the Cypher query based on the direction
        if direction == "incoming":
            query = f"""
                MATCH (n:{current}) <-[r:{relationship}]- (result)
                WHERE id(n) = {nodeId}
                RETURN result
                LIMIT 25
            """
        elif direction == "outgoing":
            query = f"""
                MATCH (n:{current}) -[r:{relationship}]-> (result)
                WHERE id(n) = {nodeId}
                RETURN result
                LIMIT 25
            """
        else:
            raise ValueError("Invalid direction: must be 'incoming' or 'outgoing'")

        result = session.run(query)
        
        nodes = []
        for record in result:
            node = record["result"]
            node_properties = dict(node.items())
            cleaned_properties = clean_node_properties(node_properties)
            node_labels = node.labels
            node_label = next(iter(node_labels), None)  # Get the first label if available
            nodes.append({"id": node.id, "properties": cleaned_properties, "label": node_label})
        return nodes

def get_distinct_relationship_types(node, node_id, db_name, driver):
    with driver.session(database=db_name) as session:
        result_incoming = session.run(f"""
            MATCH (n:{node})<-[r]-()
            WHERE id(n) = {node_id}
            RETURN DISTINCT type(r) AS relationship_type
        """)
        
        result_outgoing = session.run(f"""
            MATCH (n:{node})-[r]->()
            WHERE id(n) = {node_id}
            RETURN DISTINCT type(r) AS relationship_type
        """)
        
        incoming_relationship_types = [record["relationship_type"] for record in result_incoming]
        outgoing_relationship_types = [record["relationship_type"] for record in result_outgoing]
        
        return {
            "incoming": incoming_relationship_types or ["No incoming relationships"],
            "outgoing": outgoing_relationship_types or ["No outgoing relationships"]
        }
        
def clean_node_properties(properties):
    cleaned_properties = {}
    for key, value in properties.items():
        if isinstance(value, float) and (value != value):  # Check if the value is NaN
            cleaned_properties[key] = None
        else:
            cleaned_properties[key] = value
    return cleaned_properties




def build_batch_query(table_name, columns):
    properties = []
    for column in columns:
        default_value = "'Unknown'" if columns[column] == 'string' else "0"
        properties.append(f"{column}: COALESCE(node.{column}, {default_value})")
    
    newline = """,
"""
    properties_str = newline.join(properties)
    return f"""
    UNWIND $records AS node
    CREATE (n:{table_name.capitalize()} {{
        {properties_str}
    }})
    """

def node_to_dict(node):
    return {
        'id': node.id,
        'labels': list(node.labels),
        'properties': dict(node.items())
    }
    
def clean_node_properties(properties):
    cleaned_properties = {}
    for key, value in properties.items():
        if isinstance(value, float) and (value != value):
            cleaned_properties[key] = None
        else:
            cleaned_properties[key] = value
    return cleaned_properties



from pandas import read_sql
from neo4j.exceptions import ServiceUnavailable

def createNodesFromNodeAttribute(labelFrom, attributeName, labelTo, db_name, connection):
    query = f"""
    MATCH (n:{labelFrom})
    WITH DISTINCT n.{attributeName} AS attributeValue
    WHERE attributeValue IS NOT NULL
    MERGE (m:{labelTo} {{id: attributeValue}})
    """
    with connection.session(database=db_name) as session:
        session.run(query)


def createRelationshipBetweenNodes(labelFrom, attributeFrom, labelTo, attributeTo, relationshipType, db_name, connection):
    query = f"""
    MATCH (a:{labelFrom}), (b:{labelTo})
    WHERE a.{attributeFrom} = b.{attributeTo}
    MERGE (a)-[r:{relationshipType}]->(b)
    """
    with connection.session(database=db_name) as session:
        session.run(query)


def removeNodesWithNoRelationships(label, db_name, connection):
    query = f"""
    MATCH (n:{label})
    WHERE NOT (n)-[]-()
    DELETE n
    """
    with connection.session(database=db_name) as session:
        session.run(query)

def findOrCreateNodeByAttribute(label, attribute, value, db_name, connection):
    query = f"""
    MERGE (n:{label} {{ {attribute}: '{value}' }})
    RETURN n
    """
    with connection.session(database=db_name) as session:
        result = session.run(query)
        return result.single()[0]

def execute_neo4j_query(driver, query, parameters, db):
    with driver.session(database=db) as session:
        result = session.run(query, parameters)
    return result

def get_neo4j_databases(driver,db):
    try:
        with driver.session(database=db) as session:
            result = session.run("SHOW DATABASES")
            databases = [record["name"] for record in result]
        driver.close()
        return databases
    except ServiceUnavailable as e:
        return []
    



# setup.py
from setuptools import setup, find_packages

setup(
    name="Astryl",
    version="0.1.0",
    packages=find_packages(),
    install_requires=[
        'neo4j',
        'pandas',
        # Add other dependencies here
    ],
    author="Heinrich Schanckenberg",
    author_email="heinrich2cs@gmail.com",
    description="A library for managing Neo4j operations.",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/yourusername/my_library",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)
